@@ -6,16 +6,18 @@ use Class::C3;
 
 use HTML::FormFu::Attribute qw/ mk_attrs /;
 use HTML::FormFu::ObjectUtil qw/
-    get_error _require_constraint /;
+    get_error _require_constraint set_nested_hash_value nested_hash_key_exists
+    get_nested_hash_value /;
 use HTML::FormFu::Util qw/
-    _parse_args append_xml_attribute xml_escape require_class /;
+    _parse_args append_xml_attribute xml_escape require_class process_attrs /;
 use Storable qw/ dclone /;
 use Carp qw/ croak /;
 use Exporter qw/ import /;
 
 # used by multi.pm
 our @EXPORT_OK = qw/
-    _render_container_class _render_comment_class _render_label /;
+    _render_container_class _render_comment_class _render_label
+    _string_field_start _string_field_end _string_label /;
 
 __PACKAGE__->mk_attrs(
     qw/
@@ -29,8 +31,8 @@ __PACKAGE__->mk_accessors(
     qw/
         _constraints _filters _inflators _deflators _validators _transformers
         _errors container_tag
-        field_filename label_filename retain_default force_default
-        javascript /
+        field_filename label_filename label_tag retain_default force_default
+        javascript non_param reverse_multi multi_value original_name /
 );
 
 __PACKAGE__->mk_output_accessors(qw/ comment label value /);
@@ -38,7 +40,8 @@ __PACKAGE__->mk_output_accessors(qw/ comment label value /);
 __PACKAGE__->mk_inherited_accessors(
     qw/ auto_id auto_label auto_error_class auto_error_message
         auto_constraint_class auto_inflator_class auto_validator_class
-        auto_transformer_class render_processed_value force_errors /
+        auto_transformer_class render_processed_value force_errors
+        repeatable_count /
 );
 
 *constraints  = \&constraint;
@@ -65,13 +68,97 @@ sub new {
     $self->container_attributes( {} );
     $self->label_attributes(     {} );
     $self->label_filename('label');
+    $self->label_tag('label');
     $self->container_tag('span');
     $self->is_field(1);
-    $self->render_class_suffix('field');
 
     return $self;
 }
 
+sub name {
+    my $self = shift;
+
+    if ( @_ && $_[0] =~ /[\.\[\]]/ ) {
+        croak "field names may not contain periods or square brackets\n"
+            . "see documentation on nested_names() for details";
+    }
+
+    return $self->next::method(@_);
+}
+
+sub nested {
+    my $self = shift;
+
+    croak 'cannot set nested' if @_;
+
+    if ( defined $self->name ) {
+        while ( defined $self->parent ) {
+            $self = $self->parent;
+
+            return 1 if defined $self->nested_name;
+        }
+    }
+
+    return;
+}
+
+sub nested_name {
+    my $self = shift;
+
+    croak 'cannot set nested_name' if @_;
+
+    return if !defined $self->name;
+
+    my @names = $self->nested_names;
+
+    if ( $self->form->nested_subscript ) {
+        my $name = shift @names;
+        map { $name .= "[$_]" } @names;
+        return $name;
+    }
+    else {
+        return join ".", @names;
+    }
+}
+
+sub nested_names {
+    my $self = shift;
+
+    croak 'cannot set nested_names' if @_;
+
+    if ( defined $self->name ) {
+        my @names;
+        my $parent = $self;
+
+        while ( defined $parent->parent ) {
+            $parent = $parent->parent;
+
+            push @names, $parent->nested_name
+                if defined $parent->nested_name;
+        }
+
+        if (@names) {
+            return reverse(@names), $self->name;
+        }
+    }
+
+    return ( $self->name );
+}
+
+sub nested_base {
+    my $self = shift;
+
+    croak 'cannot set nested_base' if @_;
+
+    my $parent = $self;
+
+    while ( defined $parent->parent ) {
+        $parent = $parent->parent;
+
+        return $parent->nested_name if defined $parent->nested_name;
+    }
+}
+
 sub deflator {
     my ( $self, $arg ) = @_;
     my @return;
@@ -171,7 +258,7 @@ sub get_deflators {
     }
 
     return \@x;
-};
+}
 
 sub get_filters {
     my $self = shift;
@@ -188,7 +275,7 @@ sub get_filters {
     }
 
     return \@x;
-};
+}
 
 sub get_constraints {
     my $self = shift;
@@ -205,7 +292,7 @@ sub get_constraints {
     }
 
     return \@x;
-};
+}
 
 sub get_inflators {
     my $self = shift;
@@ -222,7 +309,7 @@ sub get_inflators {
     }
 
     return \@x;
-};
+}
 
 sub get_validators {
     my $self = shift;
@@ -239,7 +326,7 @@ sub get_validators {
     }
 
     return \@x;
-};
+}
 
 sub get_transformers {
     my $self = shift;
@@ -256,7 +343,7 @@ sub get_transformers {
     }
 
     return \@x;
-};
+}
 
 sub get_errors {
     my $self = shift;
@@ -305,21 +392,29 @@ sub process_input {
     my $submitted = $self->form->submitted;
     my $default   = $self->default;
     my $original  = $self->value;
-    my $field     = $self->name;
+    my $name      = $self->nested_name;
 
     # set input to default value (defined before calling FormFu->process)
     if ( $submitted && $self->force_default && defined $default ) {
-        $input->{$field} = $default;
+        $self->set_nested_hash_value( $input, $name, $default );
     }
+
     # checkbox, radio
     elsif ( $submitted && $self->force_default && $self->checked ) {
+
         # the checked attribute is set, so force input to be the original value
-        $input->{$field} = $original;
+        $self->set_nested_hash_value( $input, $name, $original );
     }
+
     # checkbox, radio
-    elsif ( $submitted && $self->force_default && !defined $default && defined $original ) {
-        # default and value are not equal, so this element is not checked by default
-        $input->{$field} = undef;
+    elsif ($submitted
+        && $self->force_default
+        && !defined $default
+        && defined $original )
+    {
+
+    # default and value are not equal, so this element is not checked by default
+        $self->set_nested_hash_value( $input, $name, undef );
     }
 
     return;
@@ -328,7 +423,7 @@ sub process_input {
 sub prepare_id {
     my ( $self, $render ) = @_;
 
-    if (   !defined $render->{attributes}{id}
+    if (  !defined $render->{attributes}{id}
         && defined $self->auto_id
         && length $self->auto_id )
     {
@@ -340,6 +435,10 @@ sub prepare_id {
         my $id = $self->auto_id;
         $id =~ s/%([fn])/$string{$1}/g;
 
+        if ( defined( my $count = $self->repeatable_count ) ) {
+            $id =~ s/%r/$count/g;
+        }
+
         $render->{attributes}{id} = $id;
     }
 
@@ -355,8 +454,8 @@ sub process_value {
     my $new
         = $submitted
         ? defined $value
-        ? $value
-        : defined $default ? ""
+            ? $value
+            : defined $default ? ""
         : undef
         : $default;
 
@@ -377,10 +476,11 @@ sub process_value {
     return $new;
 }
 
-sub render {
+sub render_data_non_recursive {
     my $self = shift;
 
     my $render = $self->next::method( {
+            nested_name          => xml_escape( $self->nested_name ),
             comment_attributes   => xml_escape( $self->comment_attributes ),
             container_attributes => xml_escape( $self->container_attributes ),
             label_attributes     => xml_escape( $self->label_attributes ),
@@ -388,7 +488,9 @@ sub render {
             label                => xml_escape( $self->label ),
             field_filename       => $self->field_filename,
             label_filename       => $self->label_filename,
+            label_tag            => $self->label_tag,
             container_tag        => $self->container_tag,
+            reverse_multi        => $self->reverse_multi,
             javascript           => $self->javascript,
             @_ ? %{ $_[0] } : () } );
 
@@ -416,7 +518,7 @@ sub render {
 sub _render_label {
     my ( $self, $render ) = @_;
 
-    if (   !defined $render->{label}
+    if (  !defined $render->{label}
         && defined $self->auto_label
         && length $self->auto_label )
     {
@@ -433,7 +535,7 @@ sub _render_label {
 
     if ( defined $render->{label} ) {
         append_xml_attribute( $render->{container_attributes},
-            'class', 'label' );
+            'class', $self->label_tag );
     }
 
     # label "for" attribute
@@ -463,16 +565,20 @@ sub _render_comment_class {
 sub _render_value {
     my ( $self, $render ) = @_;
 
+    my $form = $self->form;
+    my $name = $self->nested_name;
     my $render_processed;
 
     my $input
         = (    $self->form->submitted
-            && defined $self->name
-            && exists $self->form->input->{ $self->name } )
+            && defined $name
+            && $self->nested_hash_key_exists( $form->input, $name ) )
         ? $self->render_processed_value
-        ? ( $render_processed = 1
-            && $self->form->_processed_params->{ $self->name } )
-        : $self->form->input->{ $self->name }
+            ? ( $render_processed = 1
+                    && $self->get_nested_hash_value(
+                    $form->_processed_params, $name
+                    ) )
+            : $self->get_nested_hash_value( $form->input, $name )
         : undef;
 
     if ( ref $input eq 'ARRAY' ) {
@@ -635,7 +741,7 @@ sub _render_transformer_class {
 sub _render_error_class {
     my ( $self, $render ) = @_;
 
-    my @errors = @{ $self->get_errors({ forced => 1 }) };
+    my @errors = @{ $self->get_errors( { forced => 1 } ) };
 
     if (@errors) {
         $render->{errors} = \@errors;
@@ -652,6 +758,93 @@ sub _render_error_class {
     return;
 }
 
+sub render_label {
+    my ($self) = @_;
+
+    return $self->tt( { filename => $self->{label_filename} } );
+}
+
+sub render_field {
+    my ($self) = @_;
+
+    return $self->tt( { filename => $self->{field_filename} } );
+}
+
+sub _string_field_start {
+    my ( $self, $render ) = @_;
+
+    # field wrapper template - start
+
+    my $html = '';
+
+    if ( defined $render->{container_tag} ) {
+        $html .= sprintf '<%s%s>',
+            $render->{container_tag},
+            process_attrs( $render->{container_attributes} );
+    }
+
+    if ( defined $render->{label} && $render->{label_tag} eq 'legend' ) {
+        $html .= "\n" . $self->_string_label($render);
+    }
+
+    if ( $render->{errors} ) {
+        for my $error ( @{ $render->{errors} } ) {
+            $html .= "\n" . sprintf '<span class="error_message %s">%s</span>',
+                $error->class,
+                $error->message;
+        }
+    }
+
+    if ( defined $render->{label} && $render->{label_tag} ne 'legend' ) {
+        $html .= "\n" . $self->_string_label($render);
+    }
+
+    if ( defined $render->{container_tag} ) {
+        $html .= "\n";
+    }
+
+    return $html;
+}
+
+sub _string_label {
+    my ( $self, $render ) = @_;
+
+    # label template
+
+    my $html = sprintf "<%s%s>%s</%s>",
+        $render->{label_tag},
+        process_attrs( $render->{label_attributes} ),
+        $render->{label},
+        $render->{label_tag};
+
+    return $html;
+}
+
+sub _string_field_end {
+    my ( $self, $render ) = @_;
+
+    # field wrapper template - end
+
+    my $html = '';
+
+    if ( defined $render->{comment} ) {
+        $html .= sprintf "\n<span%s>\n%s\n</span>",
+            process_attrs( $render->{comment_attributes} ),
+            $render->{comment};
+    }
+
+    if ( defined $render->{container_tag} ) {
+        $html .= sprintf "\n</%s>", $render->{container_tag};
+    }
+
+    if ( defined $render->{javascript} ) {
+        $html .= sprintf 'qq{\n<script type="text/javascript">\n%s\n</script>}',
+            $render->{javascript};
+    }
+
+    return $html;
+}
+
 sub clone {
     my $self = shift;
 
@@ -659,10 +852,12 @@ sub clone {
 
     for my $list (
         qw/ _filters _constraints _inflators _validators _transformers
-        _deflators /
+        _deflators _errors /
         )
     {
         $clone->$list( [ map { $_->clone } @{ $self->$list } ] );
+
+        map { $_->parent($clone) } @{ $clone->$list };
     }
 
     $clone->comment_attributes( dclone $self->comment_attributes );
@@ -691,7 +886,7 @@ sub _single_deflator {
     push @{ $self->_deflators }, $new;
 
     return $new;
-};
+}
 
 sub _single_filter {
     my ( $self, $arg ) = @_;
@@ -712,7 +907,7 @@ sub _single_filter {
     push @{ $self->_filters }, $new;
 
     return $new;
-};
+}
 
 sub _single_constraint {
     my ( $self, $arg ) = @_;
@@ -733,7 +928,7 @@ sub _single_constraint {
     push @{ $self->_constraints }, $new;
 
     return $new;
-};
+}
 
 sub _single_inflator {
     my ( $self, $arg ) = @_;
@@ -754,7 +949,7 @@ sub _single_inflator {
     push @{ $self->_inflators }, $new;
 
     return $new;
-};
+}
 
 sub _single_validator {
     my ( $self, $arg ) = @_;
@@ -775,7 +970,7 @@ sub _single_validator {
     push @{ $self->_validators }, $new;
 
     return $new;
-};
+}
 
 sub _single_transformer {
     my ( $self, $arg ) = @_;
@@ -796,7 +991,7 @@ sub _single_transformer {
     push @{ $self->_transformers }, $new;
 
     return $new;
-};
+}
 
 1;
 
@@ -858,6 +1053,18 @@ Arguments: $localization_key
 
 Set the value using a L10N key.
 
+=head2 non_param
+
+Arguments: bool
+
+If true, values for this field are never returned by L<HTML::FormFu/params>, 
+L<HTML::FormFu/param> and L<HTML::FormFu/valid>.
+
+This is useful for Submit buttons, when you only use its value as an 
+L<indicator|HTML::FormFu/indicator>.
+
+Default Value: false
+
 =head2 label
 
 Set a label to communicate the purpose of the form-field to the user.
@@ -927,6 +1134,18 @@ input value doesn't respect that, it will remain the first value.
 
 Default Value: C<false>
 
+=head2 reverse_multi
+
+If true, then when the field is used within a 
+L<Multi|HTML::FormFu::Element::Multi> block, the field's label should be 
+rendered to the right of the field control
+
+The default value is C<false>, causing the label to be rendered to the left
+of the field control (or to be explicit: the markup for the label comes 
+before the field control in the source).
+
+Default Value: C<false>
+
 =head2 clone
 
 See L<HTML::FormFu/clone> for details.
