@@ -15,12 +15,11 @@ sub new {
 
     $self->filename('input');
     $self->field_filename('input_tag');
-    $self->multi_filename('multi_ltr');
 
     return $self;
 }
 
-sub render {
+sub render_data_non_recursive {
     my $self = shift;
 
     my $render = $self->next::method( {
@@ -30,6 +29,51 @@ sub render {
     return $render;
 }
 
+sub string {
+    my ( $self, $args ) = @_;
+
+    $args ||= {};
+
+    my $render
+        = exists $args->{render_data}
+        ? $args->{render_data}
+        : $self->render_data;
+
+    # field wrapper template - start
+
+    my $html = $self->_string_field_start($render);
+
+    # input_tag template
+
+    $html .= $self->_string_field($render);
+
+    # field wrapper template - end
+
+    $html .= $self->_string_field_end($render);
+
+    return $html;
+}
+
+sub _string_field {
+    my ( $self, $render ) = @_;
+
+    my $html .= "<input";
+
+    if ( defined $render->{nested_name} ) {
+        $html .= sprintf qq{ name="%s"}, $render->{nested_name};
+    }
+
+    $html .= sprintf qq{ type="%s"}, $render->{field_type};
+
+    if ( defined $render->{value} ) {
+        $html .= sprintf qq{ value="%s"}, $render->{value};
+    }
+
+    $html .= sprintf "%s />", process_attrs( $render->{attributes} );
+
+    return $html;
+}
+
 sub as {
     my ( $self, $type, %attrs ) = @_;
 
